@@ -41,8 +41,6 @@
  */
 #include <ldns/config.h>
 
-#include <ldns/ldns.h>
-
 #include <sys/types.h>
 #include <sys/param.h>
 #ifdef HAVE_SYS_SOCKET_H
@@ -131,7 +129,7 @@ static const char Pad64 = '=';
    */
 
 int
-b64_ntop(uint8_t const *src, size_t srclength, char *target, size_t targsize) {
+ldns_b64_ntop(uint8_t const *src, size_t srclength, char *target, size_t targsize) {
 	size_t datalength = 0;
 	uint8_t input[3];
 	uint8_t output[4];
