@@ -43,6 +43,11 @@
  * Both the server and the client(control tool) have their own keys.
  */
 #include "config.h"
+#ifdef HAVE_OPENSSL_ERR_H
+#include <openssl/err.h>
+#endif
+#include <ctype.h>
+#include "ldns/ldns.h"
 #include "daemon/remote.h"
 #include "daemon/worker.h"
 #include "daemon/daemon.h"
@@ -312,7 +317,7 @@ struct listen_port* daemon_remote_open_ports(struct config_file* cfg)
 	struct listen_port* l = NULL;
 	log_assert(cfg->remote_control_enable && cfg->control_port);
 	if(cfg->control_ifs) {
-		struct config_strlist* p = cfg->control_ifs;
+		struct config_strlist* p;
 		for(p = cfg->control_ifs; p; p = p->next) {
 			if(!add_open(p->str, cfg->control_port, &l, 1)) {
 				listening_ports_free(l);
@@ -872,6 +877,7 @@ do_stats(SSL* ssl, struct daemon_remote* rc, int reset)
 	struct stats_info total;
 	struct stats_info s;
 	int i;
+	log_assert(daemon->num > 0);
 	/* gather all thread statistics in one place */
 	for(i=0; i<daemon->num; i++) {
 		server_stats_obtain(rc->worker, daemon->workers[i], &s, reset);
@@ -1111,6 +1117,8 @@ struct del_info {
 	size_t len;
 	/** labels */
 	int labs;
+	/** now */
+	uint32_t now;
 	/** time to invalidate to */
 	uint32_t expired;
 	/** number of rrsets removed */
@@ -1131,8 +1139,10 @@ zone_del_rrset(struct lruhash_entry* e, void* arg)
 	if(dname_subdomain_c(k->rk.dname, inf->name)) {
 		struct packed_rrset_data* d = 
 			(struct packed_rrset_data*)e->data;
-		d->ttl = inf->expired;
-		inf->num_rrsets++;
+		if(d->ttl >= inf->now) {
+			d->ttl = inf->expired;
+			inf->num_rrsets++;
+		}
 	}
 }
 
@@ -1145,8 +1155,10 @@ zone_del_msg(struct lruhash_entry* e, void* arg)
 	struct msgreply_entry* k = (struct msgreply_entry*)e->key;
 	if(dname_subdomain_c(k->key.qname, inf->name)) {
 		struct reply_info* d = (struct reply_info*)e->data;
-		d->ttl = inf->expired;
-		inf->num_msgs++;
+		if(d->ttl >= inf->now) {
+			d->ttl = inf->expired;
+			inf->num_msgs++;
+		}
 	}
 }
 
@@ -1159,8 +1171,10 @@ zone_del_kcache(struct lruhash_entry* e, void* arg)
 	struct key_entry_key* k = (struct key_entry_key*)e->key;
 	if(dname_subdomain_c(k->name, inf->name)) {
 		struct key_entry_data* d = (struct key_entry_data*)e->data;
-		d->ttl = inf->expired;
-		inf->num_keys++;
+		if(d->ttl >= inf->now) {
+			d->ttl = inf->expired;
+			inf->num_keys++;
+		}
 	}
 }
 
@@ -1180,6 +1194,7 @@ do_flush_zone(SSL* ssl, struct worker* worker, char* arg)
 	inf.name = nm;
 	inf.len = nmlen;
 	inf.labs = nmlabs;
+	inf.now = *worker->env.now;
 	inf.expired = *worker->env.now;
 	inf.expired -= 3; /* handle 3 seconds skew between threads */
 	inf.num_rrsets = 0;
@@ -1603,7 +1618,7 @@ do_list_local_data(SSL* ssl, struct worker* worker)
 }
 
 /** tell other processes to execute the command */
-void
+static void
 distribute_cmd(struct daemon_remote* rc, SSL* ssl, char* cmd)
 {
 	int i;
@@ -1787,8 +1802,8 @@ int remote_control_callback(struct comm_point* c, void* arg, int err,
 	ERR_clear_error();
 	r = SSL_do_handshake(s->ssl);
 	if(r != 1) {
-		r = SSL_get_error(s->ssl, r);
-		if(r == SSL_ERROR_WANT_READ) {
+		int r2 = SSL_get_error(s->ssl, r);
+		if(r2 == SSL_ERROR_WANT_READ) {
 			if(s->shake_state == rc_hs_read) {
 				/* try again later */
 				return 0;
@@ -1796,7 +1811,7 @@ int remote_control_callback(struct comm_point* c, void* arg, int err,
 			s->shake_state = rc_hs_read;
 			comm_point_listen_for_rw(c, 1, 0);
 			return 0;
-		} else if(r == SSL_ERROR_WANT_WRITE) {
+		} else if(r2 == SSL_ERROR_WANT_WRITE) {
 			if(s->shake_state == rc_hs_write) {
 				/* try again later */
 				return 0;
@@ -1805,6 +1820,10 @@ int remote_control_callback(struct comm_point* c, void* arg, int err,
 			comm_point_listen_for_rw(c, 0, 1);
 			return 0;
 		} else {
+			if(r == 0)
+				log_err("remote control connection closed prematurely");
+			log_addr(1, "failed connection from",
+				&s->c->repinfo.addr, s->c->repinfo.addrlen);
 			log_crypto_err("remote control failed ssl");
 			clean_point(rc, s);
 			return 0;
