@@ -42,7 +42,7 @@ ldns_dnssec_get_rrsig_for_name_and_type(const ldns_rdf *name,
 		if (ldns_rr_get_type(candidate) == LDNS_RR_TYPE_RRSIG) {
 			if (ldns_dname_compare(ldns_rr_owner(candidate),
 			                       name) == 0 &&
-			    ldns_rdf2native_int8(ldns_rr_rrsig_typecovered(candidate))
+			    ldns_rdf2rr_type(ldns_rr_rrsig_typecovered(candidate))
 			    == type
 			    ) {
 				return candidate;
@@ -148,6 +148,12 @@ ldns_dnssec_nsec3_closest_encloser(ldns_rdf *qname,
 									 salt);
 
 		status = ldns_dname_cat(hashed_sname, zone_name);
+                if(status != LDNS_STATUS_OK) {
+	                LDNS_FREE(salt);
+	                ldns_rdf_deep_free(zone_name);
+	                ldns_rdf_deep_free(sname);
+                        return NULL;
+                }
 
 		for (nsec_i = 0; nsec_i < ldns_rr_list_rr_count(nsec3s); nsec_i++) {
 			nsec = ldns_rr_list_rr(nsec3s, nsec_i);
@@ -166,6 +172,9 @@ ldns_dnssec_nsec3_closest_encloser(ldns_rdf *qname,
 			flag = true;
 		} else if (exact_match_found && flag) {
 			result = ldns_rdf_clone(sname);
+			/* RFC 5155: 8.3. 2.** "The proof is complete" */
+			ldns_rdf_deep_free(hashed_sname);
+			goto done;
 		} else if (exact_match_found && !flag) {
 			/* error! */
 			ldns_rdf_deep_free(hashed_sname);
@@ -503,7 +512,7 @@ ldns_key_rr2ds(const ldns_rr *key, ldns_hash h)
 			return NULL;
 		}
 		break;
-	case LDNS_HASH_GOST94:
+	case LDNS_HASH_GOST:
 #ifdef USE_GOST
 		(void)ldns_key_EVP_load_gost_id();
 		md = EVP_get_digestbyname("md_gost94");
@@ -590,7 +599,7 @@ ldns_key_rr2ds(const ldns_rr *key, ldns_hash h)
 		                            digest);
 		ldns_rr_push_rdf(ds, tmp);
 		break;
-	case LDNS_HASH_GOST94:
+	case LDNS_HASH_GOST:
 #ifdef USE_GOST
 		if(!ldns_digest_evp((unsigned char *) ldns_buffer_begin(data_buf),
 				(unsigned int) ldns_buffer_position(data_buf),
@@ -804,6 +813,10 @@ ldns_dnssec_create_nsec3(ldns_dnssec_name *from,
 	                  salt_length,
 	                  salt));
 	status = ldns_dname_cat(ldns_rr_owner(nsec_rr), zone_name);
+        if(status != LDNS_STATUS_OK) {
+                ldns_rr_free(nsec_rr);
+                return NULL;
+        }
 	ldns_nsec3_add_param_rdfs(nsec_rr,
 	                          algorithm,
 	                          flags,
@@ -902,8 +915,8 @@ ldns_nsec3_hash_name(ldns_rdf *name,
 				 uint8_t salt_length,
 				 uint8_t *salt)
 {
-	char *orig_owner_str;
 	size_t hashed_owner_str_len;
+	ldns_rdf *cann;
 	ldns_rdf *hashed_owner;
 	unsigned char *hashed_owner_str;
 	char *hashed_owner_b32;
@@ -915,15 +928,21 @@ ldns_nsec3_hash_name(ldns_rdf *name,
 	ldns_status status;
 
 	/* prepare the owner name according to the draft section bla */
-	orig_owner_str = ldns_rdf2str(name);
+	cann = ldns_rdf_clone(name);
+	if(!cann) {
+		fprintf(stderr, "Memory error\n");
+		return NULL;
+	}
+	ldns_dname2canonical(cann);
 
 	/* TODO: mnemonic list for hash algs SHA-1, default to 1 now (sha1) */
 	algorithm = algorithm;
 
-	hashed_owner_str_len = salt_length + ldns_rdf_size(name);
+	hashed_owner_str_len = salt_length + ldns_rdf_size(cann);
 	hashed_owner_str = LDNS_XMALLOC(unsigned char, hashed_owner_str_len);
-	memcpy(hashed_owner_str, ldns_rdf_data(name), ldns_rdf_size(name));
-	memcpy(hashed_owner_str + ldns_rdf_size(name), salt, salt_length);
+	memcpy(hashed_owner_str, ldns_rdf_data(cann), ldns_rdf_size(cann));
+	memcpy(hashed_owner_str + ldns_rdf_size(cann), salt, salt_length);
+	ldns_rdf_deep_free(cann);
 
 	for (cur_it = iterations + 1; cur_it > 0; cur_it--) {
 		(void) ldns_sha1((unsigned char *) hashed_owner_str,
@@ -934,27 +953,24 @@ ldns_nsec3_hash_name(ldns_rdf *name,
 		hashed_owner_str = LDNS_XMALLOC(unsigned char, hashed_owner_str_len);
 		if (!hashed_owner_str) {
 			fprintf(stderr, "Memory error\n");
-			abort();
+			return NULL;
 		}
 		memcpy(hashed_owner_str, hash, LDNS_SHA1_DIGEST_LENGTH);
 		memcpy(hashed_owner_str + LDNS_SHA1_DIGEST_LENGTH, salt, salt_length);
 		hashed_owner_str_len = LDNS_SHA1_DIGEST_LENGTH + salt_length;
 	}
 
-	LDNS_FREE(orig_owner_str);
 	LDNS_FREE(hashed_owner_str);
 	hashed_owner_str = hash;
 	hashed_owner_str_len = LDNS_SHA1_DIGEST_LENGTH;
 
 	hashed_owner_b32 = LDNS_XMALLOC(char,
-							  ldns_b32_ntop_calculate_size(
-							       hashed_owner_str_len) + 1);
-	hashed_owner_b32_len =
-		(size_t) ldns_b32_ntop_extended_hex((uint8_t *) hashed_owner_str,
-									 hashed_owner_str_len,
-									 hashed_owner_b32,
-									 ldns_b32_ntop_calculate_size(
-										 hashed_owner_str_len));
+                  ldns_b32_ntop_calculate_size(hashed_owner_str_len) + 1);
+        hashed_owner_b32_len = (size_t) ldns_b32_ntop_extended_hex(
+                (uint8_t *) hashed_owner_str,
+                hashed_owner_str_len,
+                hashed_owner_b32,
+                ldns_b32_ntop_calculate_size(hashed_owner_str_len));
 	if (hashed_owner_b32_len < 1) {
 		fprintf(stderr, "Error in base32 extended hex encoding ");
 		fprintf(stderr, "of hashed owner name (name: ");
@@ -964,7 +980,6 @@ ldns_nsec3_hash_name(ldns_rdf *name,
 		LDNS_FREE(hashed_owner_b32);
 		return NULL;
 	}
-	hashed_owner_str_len = hashed_owner_b32_len;
 	hashed_owner_b32[hashed_owner_b32_len] = '\0';
 
 	status = ldns_str2rdf_dname(&hashed_owner, hashed_owner_b32);
@@ -1069,8 +1084,12 @@ ldns_create_nsec3(ldns_rdf *cur_owner,
 								 salt_length,
 								 salt);
 	status = ldns_dname_cat(hashed_owner, cur_zone);
+        if(status != LDNS_STATUS_OK)
+                return NULL;
 
 	nsec = ldns_rr_new_frm_type(LDNS_RR_TYPE_NSEC3);
+        if(!nsec)
+                return NULL;
 	ldns_rr_set_type(nsec, LDNS_RR_TYPE_NSEC3);
 	ldns_rr_set_owner(nsec, hashed_owner);
 
@@ -1524,14 +1543,14 @@ ldns_convert_dsa_rrsig_rdf2asn1(ldns_buffer *target_buffer,
 						  const ldns_rdf *sig_rdf)
 {
 	/* the EVP api wants the DER encoding of the signature... */
-	uint8_t t;
 	BIGNUM *R, *S;
 	DSA_SIG *dsasig;
 	unsigned char *raw_sig = NULL;
 	int raw_sig_len;
 
+        if(ldns_rdf_size(sig_rdf) < 1 + 2*SHA_DIGEST_LENGTH)
+                return LDNS_STATUS_SYNTAX_RDATA_ERR;
 	/* extract the R and S field from the sig buffer */
-	t = ldns_rdf_data(sig_rdf)[0];
 	R = BN_new();
 	if(!R) return LDNS_STATUS_MEM_ERR;
 	(void) BN_bin2bn((unsigned char *) ldns_rdf_data(sig_rdf) + 1,
