@@ -31,6 +31,9 @@ ldns_create_empty_rrsig(ldns_rr_list *rrset,
 
 	label_count = ldns_dname_label_count(ldns_rr_owner(ldns_rr_list_rr(rrset,
 	                                                   0)));
+        /* RFC4035 2.2: not counting the leftmost label if it is a wildcard */
+        if(ldns_dname_is_wildcard(ldns_rr_owner(ldns_rr_list_rr(rrset, 0))))
+                label_count --; 
 	
 	current_sig = ldns_rr_new_frm_type(LDNS_RR_TYPE_RRSIG);
 	
@@ -145,7 +148,7 @@ ldns_sign_public_buffer(ldns_buffer *sign_buf, ldns_key *current_key)
 		break;
 #endif /* USE_SHA2 */
 #ifdef USE_GOST
-	case LDNS_SIGN_GOST:
+	case LDNS_SIGN_ECC_GOST:
 		b64rdf = ldns_sign_public_evp(
 				   sign_buf,
 				   ldns_key_evp_key(current_key),
@@ -191,7 +194,6 @@ ldns_sign_public(ldns_rr_list *rrset, ldns_key_list *keys)
 	
 	new_owner = NULL;
 
-	key_count = 0;
 	signatures = ldns_rr_list_new();
 
 	/* prepare a signature and add all the know data
@@ -707,7 +709,6 @@ ldns_dnssec_remove_signatures(ldns_dnssec_rrs *signatures,
 
 	uint16_t keytag;
 	size_t i;
-	int v;
 
 	key_list = key_list;
 
@@ -726,7 +727,7 @@ ldns_dnssec_remove_signatures(ldns_dnssec_rrs *signatures,
 		}
 		return NULL;
 	}
-	v = func(cur_rr->rr, arg);
+	(void)func(cur_rr->rr, arg);
 
 	while (cur_rr) {
 		next_rr = cur_rr->next;
@@ -1093,7 +1094,7 @@ ldns_zone_sign(const ldns_zone *zone, ldns_key_list *key_list)
 	dnssec_zone = ldns_dnssec_zone_new();
 
 	(void) ldns_dnssec_zone_add_rr(dnssec_zone, ldns_zone_soa(zone));
-	ldns_zone_set_soa(signed_zone, ldns_zone_soa(zone));
+	ldns_zone_set_soa(signed_zone, ldns_rr_clone(ldns_zone_soa(zone)));
 	
 	for (i = 0; i < ldns_rr_list_rr_count(ldns_zone_rrs(zone)); i++) {
 		(void) ldns_dnssec_zone_add_rr(dnssec_zone,
@@ -1134,7 +1135,7 @@ ldns_zone_sign_nsec3(ldns_zone *zone, ldns_key_list *key_list, uint8_t algorithm
 	dnssec_zone = ldns_dnssec_zone_new();
 
 	(void) ldns_dnssec_zone_add_rr(dnssec_zone, ldns_zone_soa(zone));
-	ldns_zone_set_soa(signed_zone, ldns_zone_soa(zone));
+	ldns_zone_set_soa(signed_zone, ldns_rr_clone(ldns_zone_soa(zone)));
 	
 	for (i = 0; i < ldns_rr_list_rr_count(ldns_zone_rrs(zone)); i++) {
 		(void) ldns_dnssec_zone_add_rr(dnssec_zone,
