@@ -12,7 +12,7 @@
 
 int verbosity = 0;
 
-int
+static int
 usage(FILE *fp, char *prog) {
 	fprintf(fp, "%s [options] domain\n", prog);
 	fprintf(fp, "  print out the owner names for domain and the record types for those names\n");
@@ -159,7 +159,7 @@ query_type_bitmaps(ldns_resolver *res,
 				    if (verbosity >=  3) {
 						printf("querying for:\n");
 						ldns_rdf_print(stdout, name);
-						printf(" type %u\n", type);
+						printf(" type %u\n", (unsigned int) type);
 					}
 					answer_pkt = ldns_resolver_query(res, name, type,
 					                                 LDNS_RR_CLASS_IN,
@@ -453,7 +453,7 @@ main(int argc, char *argv[])
         }
 
 	/* add \001 to soa */
-	status = ldns_str2rdf_dname(&soa_p1, "0");
+	status = ldns_str2rdf_dname(&soa_p1, "\\000");
 	if (status != LDNS_STATUS_OK) {
 		printf("error: %s\n", ldns_get_errorstr_by_id(status));
 	}
@@ -619,7 +619,6 @@ main(int argc, char *argv[])
 		last_dname_p = create_plus_1_dname(last_dname);
 	} else {
 		if (last_dname) {
-			/*ldns_rdf_print(stdout, last_dname);*/
 			if (ldns_rdf_compare(last_dname, next_dname) == 0) {
 				printf("\n\nNext dname is the same as current, this would loop forever. This is a problem that usually occurs when walking through a caching forwarder. Try using the authoritative nameserver to walk (with @nameserver).\n");
 				exit(2);
@@ -637,10 +636,10 @@ main(int argc, char *argv[])
 			ldns_rdf_print(stdout, ldns_rr_rdf(nsec_rr, 1));
 			printf("\n");
 		} else {
-			/* ok, so now we now all the types present at this name,
+			/* ok, so now we know all the types present at this name,
 			 * query for those one by one (...)
 			 */
-			query_type_bitmaps(res, LDNS_RD, ldns_rr_owner(nsec_rr),
+			status = query_type_bitmaps(res, LDNS_RD, ldns_rr_owner(nsec_rr),
 			                   ldns_rr_rdf(nsec_rr, 1));
 			/* print this nsec and its signatures too */
 			ldns_rr_print(stdout, nsec_rr);
