@@ -542,19 +542,19 @@ ldns_resolver_push_searchlist(ldns_resolver *r, ldns_rdf *d)
 void
 ldns_resolver_set_tsig_keyname(ldns_resolver *r, char *tsig_keyname)
 {
-	r->_tsig_keyname = tsig_keyname;
+	r->_tsig_keyname = strdup(tsig_keyname);
 }
 
 void
 ldns_resolver_set_tsig_algorithm(ldns_resolver *r, char *tsig_algorithm)
 {
-	r->_tsig_algorithm = tsig_algorithm;
+	r->_tsig_algorithm = strdup(tsig_algorithm);
 }
 
 void
 ldns_resolver_set_tsig_keydata(ldns_resolver *r, char *tsig_keydata)
 {
-	r->_tsig_keydata = tsig_keydata;
+	r->_tsig_keydata = strdup(tsig_keydata);
 }
 
 void
@@ -594,6 +594,9 @@ ldns_resolver_new(void)
 	ldns_resolver_set_dnssec_cd(r, false);
 	ldns_resolver_set_dnssec_anchors(r, NULL);
 	ldns_resolver_set_ip6(r, LDNS_RESOLV_INETANY);
+	ldns_resolver_set_igntc(r, false);
+	ldns_resolver_set_recursive(r, false);
+	ldns_resolver_set_dnsrch(r, true);
 
 	/* randomize the nameserver to be queried
 	 * when there are multiple
@@ -636,6 +639,8 @@ ldns_resolver_new_frm_fp_l(ldns_resolver **res, FILE *fp, int *line_nr)
 #endif
 	ssize_t gtr;
 	ldns_buffer *b;
+        int lnr = 0, oldline;
+        if(!line_nr) line_nr = &lnr;
 
 	/* do this better
 	 * expect =
@@ -661,25 +666,36 @@ ldns_resolver_new_frm_fp_l(ldns_resolver **res, FILE *fp, int *line_nr)
 
 	gtr = 1;
 	word[0] = 0;
+        oldline = *line_nr;
+        expect = LDNS_RESOLV_KEYWORD;
 	while (gtr > 0) {
 		/* check comments */
 		if (word[0] == '#') {
-			/* read the rest of the line, should be 1 word */
-			gtr = ldns_fget_token_l(fp, word, LDNS_PARSE_NORMAL, 0, line_nr);
-			/* prepare the next string for further parsing */
-			gtr = ldns_fget_token_l(fp, word, LDNS_PARSE_NORMAL, 0, line_nr);
+                        word[0]='x';
+                        if(oldline == *line_nr) {
+                                /* skip until end of line */
+                                int c;
+                                do {
+                                        c = fgetc(fp);
+                                } while(c != EOF && c != '\n');
+                                if(c=='\n' && line_nr) (*line_nr)++;
+                        }
+			/* and read next to prepare for further parsing */
+                        oldline = *line_nr;
 			continue;
 		}
+                oldline = *line_nr;
 		switch(expect) {
 			case LDNS_RESOLV_KEYWORD:
 				/* keyword */
 				gtr = ldns_fget_token_l(fp, word, LDNS_PARSE_NORMAL, 0, line_nr);
 				if (gtr != 0) {
+                                        if(word[0] == '#') continue;
 					for(i = 0; i < LDNS_RESOLV_KEYWORDS; i++) {
 						if (strcasecmp(keyword[i], word) == 0) {
 							/* chosen the keyword and
 							 * expect values carefully
-							 */
+	        					 */
 							expect = i;
 							break;
 						}
@@ -700,6 +716,10 @@ ldns_resolver_new_frm_fp_l(ldns_resolver **res, FILE *fp, int *line_nr)
 				if (gtr == 0) {
 					return LDNS_STATUS_SYNTAX_MISSING_VALUE_ERR;
 				}
+                                if(word[0] == '#') {
+                                        expect = LDNS_RESOLV_KEYWORD;
+                                        continue;
+                                }
 				tmp = ldns_rdf_new_frm_str(LDNS_RDF_TYPE_DNAME, word);
 				if (!tmp) {
 					ldns_resolver_deep_free(r);
@@ -716,6 +736,10 @@ ldns_resolver_new_frm_fp_l(ldns_resolver **res, FILE *fp, int *line_nr)
 				if (gtr == 0) {
 					return LDNS_STATUS_SYNTAX_MISSING_VALUE_ERR;
 				}
+                                if(word[0] == '#') {
+                                        expect = LDNS_RESOLV_KEYWORD;
+                                        continue;
+                                }
 				tmp = ldns_rdf_new_frm_str(LDNS_RDF_TYPE_AAAA, word);
 				if (!tmp) {
 					/* try ip4 */
@@ -734,10 +758,14 @@ ldns_resolver_new_frm_fp_l(ldns_resolver **res, FILE *fp, int *line_nr)
 				/* search list domain dname */
 				gtr = ldns_fget_token_l(fp, word, LDNS_PARSE_SKIP_SPACE, 0, line_nr);
 				b = LDNS_MALLOC(ldns_buffer);
-				ldns_buffer_new_frm_data(b, word, (size_t) gtr);
 
-				gtr = ldns_bget_token(b, word, LDNS_PARSE_NORMAL, (size_t) gtr);
+				ldns_buffer_new_frm_data(b, word, (size_t) gtr);
+				gtr = ldns_bget_token(b, word, LDNS_PARSE_NORMAL, (size_t) gtr + 1);
 				while (gtr > 0) {
+                                        if(word[0] == '#') {
+                                                expect = LDNS_RESOLV_KEYWORD;
+                                                continue;
+                                        }
 					tmp = ldns_rdf_new_frm_str(LDNS_RDF_TYPE_DNAME, word);
 					if (!tmp) {
 						ldns_resolver_deep_free(r);
@@ -747,7 +775,7 @@ ldns_resolver_new_frm_fp_l(ldns_resolver **res, FILE *fp, int *line_nr)
 					ldns_resolver_push_searchlist(r, tmp);
 
 					ldns_rdf_deep_free(tmp);
-					gtr = ldns_bget_token(b, word, LDNS_PARSE_NORMAL, (size_t) gtr);
+					gtr = ldns_bget_token(b, word, LDNS_PARSE_NORMAL, (size_t) gtr + 1);
 				}
 				ldns_buffer_free(b);
 				gtr = 1;
@@ -769,6 +797,10 @@ ldns_resolver_new_frm_fp_l(ldns_resolver **res, FILE *fp, int *line_nr)
 				if (gtr == 0) {
 					return LDNS_STATUS_SYNTAX_MISSING_VALUE_ERR;
 				}
+                                if(word[0] == '#') {
+                                        expect = LDNS_RESOLV_KEYWORD;
+                                        continue;
+                                }
 
 #ifdef HAVE_SSL
 				tmp_rr = ldns_read_anchor_file(word);
@@ -845,9 +877,15 @@ ldns_resolver_deep_free(ldns_resolver *res)
 		if (ldns_resolver_domain(res)) {
 			ldns_rdf_deep_free(ldns_resolver_domain(res));
 		}
-		if (ldns_resolver_tsig_keyname(res)) {
+		if (res->_tsig_keyname) {
 			LDNS_FREE(res->_tsig_keyname);
 		}
+		if (res->_tsig_keydata) {
+			LDNS_FREE(res->_tsig_keydata);
+		}
+		if (res->_tsig_algorithm) {
+			LDNS_FREE(res->_tsig_algorithm);
+		}
 
 		if (res->_cur_axfr_pkt) {
 			ldns_pkt_free(res->_cur_axfr_pkt);
@@ -864,7 +902,7 @@ ldns_resolver_deep_free(ldns_resolver *res)
 }
 
 ldns_pkt *
-ldns_resolver_search(const ldns_resolver *r,const  ldns_rdf *name, 
+ldns_resolver_search(const ldns_resolver *r,const  ldns_rdf *name,
 	ldns_rr_type t, ldns_rr_class c, uint16_t flags)
 {
 
@@ -879,7 +917,7 @@ ldns_resolver_search(const ldns_resolver *r,const  ldns_rdf *name,
 	if (ldns_dname_str_absolute(str_dname)) {
 		/* query as-is */
 		return ldns_resolver_query(r, name, t, c, flags);
-	} else {
+	} else if (ldns_resolver_dnsrch(r)) {
 		search_list = ldns_resolver_searchlist(r);
 		for (i = 0; i < ldns_resolver_searchlist_count(r); i++) {
 			new_name = ldns_dname_cat_clone(name, search_list[i]);
@@ -913,7 +951,6 @@ ldns_resolver_query(const ldns_resolver *r, const ldns_rdf *name,
 			if (pkt) {
 				ldns_pkt_free(pkt);
 			}
-			fprintf(stderr, "error: %s\n", ldns_get_errorstr_by_id(status));
 			return NULL;
 		}
 	}
@@ -940,7 +977,7 @@ ldns_resolver_query(const ldns_resolver *r, const ldns_rdf *name,
 		return NULL;
 	}
 
-	status = ldns_resolver_send(&pkt, (ldns_resolver *)r, newname, t, c,
+	(void)ldns_resolver_send(&pkt, (ldns_resolver *)r, newname, t, c,
 			flags);
 
 	ldns_rdf_free(newname);
@@ -1174,16 +1211,15 @@ ldns_axfr_last_pkt(const ldns_resolver *res)
 void
 ldns_resolver_nameservers_randomize(ldns_resolver *r)
 {
-	uint8_t i, j;
+	uint16_t i, j;
 	ldns_rdf **ns, *tmp;
 
 	/* should I check for ldns_resolver_random?? */
 	assert(r != NULL);
 
 	ns = ldns_resolver_nameservers(r);
-
 	for (i = 0; i < ldns_resolver_nameserver_count(r); i++) {
-		j = random() % ldns_resolver_nameserver_count(r);
+		j = ldns_get_random() % ldns_resolver_nameserver_count(r);
 		tmp = ns[i];
 		ns[i] = ns[j];
 		ns[j] = tmp;
