@@ -145,9 +145,6 @@ ldns_rr_new_frm_str_internal(ldns_rr **newrr, const char *str,
 	if (!new || !owner || !ttl || !clas || !rdata || !rr_buf || !rd_buf || !rd || !b64 ) {
 		return LDNS_STATUS_MEM_ERR;
 	}
-	r_cnt = 0;
-	ttl_val = 0;
-	clas_val = 0;
 
 	ldns_buffer_new_frm_data(rr_buf, (char*)str, strlen(str));
 
@@ -394,7 +391,7 @@ ldns_rr_new_frm_str_internal(ldns_rr **newrr, const char *str,
 					ldns_buffer_skip(rd_buf, 1);
 				}
 
-                                pre_data_pos = ldns_buffer_position(rd_buf);
+				pre_data_pos = ldns_buffer_position(rd_buf);
 				if ((c = ldns_bget_token(rd_buf, rd, delimiters,
 							LDNS_MAX_RDFLEN)) != -1) {
 					/* hmmz, rfc3597 specifies that any type can be represented with
@@ -442,8 +439,8 @@ ldns_rr_new_frm_str_internal(ldns_rr **newrr, const char *str,
 						/* correct the rdf type */
 						/* if *we* know the type, interpret it as wireformat */
 						if (desc) {
-                                                        size_t hex_pos = 0;
-                                                        uint8_t *hex_data = LDNS_XMALLOC(uint8_t, hex_data_size + 2);
+							size_t hex_pos = 0;
+							uint8_t *hex_data = LDNS_XMALLOC(uint8_t, hex_data_size + 2);
 							ldns_write_uint16(hex_data, hex_data_size);
 							ldns_hexstring_to_data(hex_data + 2, hex_data_str);
 							(void) ldns_wire2rdf(new, hex_data,
@@ -591,7 +588,6 @@ ldns_rr_new_frm_fp_l(ldns_rr **newrr, FILE *fp, uint32_t *default_ttl, ldns_rdf
 	ldns_status s;
 	ssize_t size;
 
-	s = LDNS_STATUS_ERR;
 	if (default_ttl) {
 		ttl = *default_ttl;
 	} else {
@@ -639,7 +635,7 @@ ldns_rr_new_frm_fp_l(ldns_rr **newrr, FILE *fp, uint32_t *default_ttl, ldns_rdf
 			*default_ttl = ldns_str2period(keyword + 5, &endptr);
 		}
 		s = LDNS_STATUS_SYNTAX_TTL;
-	} else if ((keyword = strstr(line, "$INCLUDE "))) {
+	} else if (strstr(line, "$INCLUDE ")) {
 		s = LDNS_STATUS_SYNTAX_INCLUDE;
 	} else {
 		if (origin && *origin) {
@@ -696,11 +692,9 @@ ldns_rr_set_rdf(ldns_rr *rr, const ldns_rdf *f, size_t position)
 {
 	size_t rd_count;
 	ldns_rdf *pop;
-	ldns_rdf **rdata_fields;
 
 	rd_count = ldns_rr_rd_count(rr);
 	if (position < rd_count) {
-		rdata_fields = rr->_rdata_fields;
 		/* dicard the old one */
 		pop = rr->_rdata_fields[position];
 		rr->_rdata_fields[position] = (ldns_rdf*)f;
@@ -738,6 +732,7 @@ ldns_rr_pop_rdf(ldns_rr *rr)
 {
 	size_t rd_count;
 	ldns_rdf *pop;
+	ldns_rdf** newrd;
 
 	rd_count = ldns_rr_rd_count(rr);
 
@@ -747,9 +742,15 @@ ldns_rr_pop_rdf(ldns_rr *rr)
 
 	pop = rr->_rdata_fields[rd_count - 1];
 
-	/* shrink the array */
-	rr->_rdata_fields = LDNS_XREALLOC(
-		rr->_rdata_fields, ldns_rdf *, rd_count - 1);
+	/* try to shrink the array */
+	if(rd_count > 1) {
+		newrd = LDNS_XREALLOC(
+			rr->_rdata_fields, ldns_rdf *, rd_count - 1);
+		if(newrd)
+			rr->_rdata_fields = newrd;
+	} else {
+		LDNS_FREE(rr->_rdata_fields);
+	}
 
 	ldns_rr_set_rd_count(rr, rd_count - 1);
 	return pop;
@@ -885,12 +886,9 @@ bool
 ldns_rr_list_cat(ldns_rr_list *left, ldns_rr_list *right)
 {
 	size_t r_rr_count;
-	size_t l_rr_count;
 	size_t i;
 
-	if (left) {
-		l_rr_count = ldns_rr_list_rr_count(left);
-	} else {
+	if (!left) {
 		return false;
 	}
 
@@ -915,8 +913,6 @@ ldns_rr_list_cat_clone(ldns_rr_list *left, ldns_rr_list *right)
 	size_t i;
 	ldns_rr_list *cat;
 
-	l_rr_count = 0;
-
 	if (left) {
 		l_rr_count = ldns_rr_list_rr_count(left);
 	} else {
@@ -1264,6 +1260,7 @@ ldns_rr_clone(const ldns_rr *rr)
 	ldns_rr_set_ttl(new_rr, ldns_rr_ttl(rr));
 	ldns_rr_set_type(new_rr, ldns_rr_get_type(rr));
 	ldns_rr_set_class(new_rr, ldns_rr_get_class(rr));
+	ldns_rr_set_question(new_rr, ldns_rr_is_question(rr));
 
 	for (i = 0; i < ldns_rr_rd_count(rr); i++) {
         	if (ldns_rr_rdf(rr,i)) {
@@ -1808,6 +1805,9 @@ static const ldns_rdf_type type_nsec_wireformat[] = {
 static const ldns_rdf_type type_dhcid_wireformat[] = {
 	LDNS_RDF_TYPE_B64
 };
+static const ldns_rdf_type type_talink_wireformat[] = {
+	LDNS_RDF_TYPE_DNAME, LDNS_RDF_TYPE_DNAME
+};
 /* nsec3 is some vars, followed by same type of data of nsec */
 static const ldns_rdf_type type_nsec3_wireformat[] = {
 /*	LDNS_RDF_TYPE_NSEC3_VARS, LDNS_RDF_TYPE_NSEC3_NEXT_OWNER, LDNS_RDF_TYPE_NSEC*/
@@ -1956,7 +1956,7 @@ static ldns_rr_descriptor rdata_field_descriptors[] = {
 {LDNS_RR_TYPE_NULL, "TYPE55", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 {LDNS_RR_TYPE_NULL, "TYPE56", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 {LDNS_RR_TYPE_NULL, "TYPE57", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
-{LDNS_RR_TYPE_NULL, "TYPE58", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
+{LDNS_RR_TYPE_TALINK, "TALINK", 2, 2, type_talink_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 2 },
 {LDNS_RR_TYPE_NULL, "TYPE59", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 {LDNS_RR_TYPE_NULL, "TYPE60", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 {LDNS_RR_TYPE_NULL, "TYPE61", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
