@@ -52,12 +52,8 @@ ldns_rr_set_function(ldns_rr_type type, ldns_rr *rr, ldns_rdf *rdf, size_t pos)
                 return false;
         }
         pop = ldns_rr_set_rdf(rr, rdf, pos);
-        if (pop) {
- 		LDNS_FREE(pop);
-                return true;
-        } else {
-                return true;
-        }
+ 	ldns_rdf_deep_free(pop);
+        return true;
 }
 
 /* A/AAAA records */
@@ -315,6 +311,11 @@ ldns_rr_dnskey_key_size_raw(const unsigned char* keydata,
 			return 0;
 		}
 		break;
+#ifdef USE_GOST
+	case LDNS_SIGN_ECC_GOST:
+		return 512;
+		break;
+#endif
 	case LDNS_SIGN_HMACMD5:
 		return len;
 		break;
