@@ -37,6 +37,7 @@
  * Routines for message parsing a packet buffer to a descriptive structure.
  */
 #include "config.h"
+#include "ldns/ldns.h"
 #include "util/data/msgparse.h"
 #include "util/net_help.h"
 #include "util/data/dname.h"
@@ -334,16 +335,20 @@ moveover_rrsigs(ldns_buffer* pkt, struct regional* region,
 	struct rr_parse* sig = sigset->rr_first;
 	struct rr_parse* prev = NULL;
 	struct rr_parse* insert;
+	struct rr_parse* nextsig;
 	while(sig) {
+		nextsig = sig->next;
 		if(pkt_rrsig_covered_equals(pkt, sig->ttl_data, 
 			dataset->type)) {
 			if(duplicate) {
 				/* new */
 				insert = (struct rr_parse*)regional_alloc(
 					region, sizeof(struct rr_parse));
+				if(!insert) return 0;
 				insert->outside_packet = 0;
 				insert->ttl_data = sig->ttl_data;
 				insert->size = sig->size;
+				/* prev not used */
 			} else {
 				/* remove from sigset */
 				if(prev) prev->next = sig->next;
@@ -353,6 +358,7 @@ moveover_rrsigs(ldns_buffer* pkt, struct regional* region,
 				sigset->rr_count--;
 				sigset->size -= sig->size;
 				insert = sig;
+				/* prev not changed */
 			}
 			/* add to dataset */
 			dataset->rrsig_count++;
@@ -362,9 +368,10 @@ moveover_rrsigs(ldns_buffer* pkt, struct regional* region,
 			else	dataset->rrsig_first = insert;
 			dataset->rrsig_last = insert;
 			dataset->size += insert->size;
+		} else  {
+			prev = sig;
 		}
-		prev = sig;
-		sig = sig->next;
+		sig = nextsig;
 	}
 	return 1;
 }
@@ -942,7 +949,7 @@ parse_extract_edns(struct msg_parse* msg, struct edns_data* edns)
 			|| found->dname[0] != 0) return LDNS_RCODE_FORMERR; 
 		if(found->rr_count != 1) return LDNS_RCODE_FORMERR; 
 	}
-	log_assert(found->rr_first == found->rr_last && found->rr_first);
+	log_assert(found->rr_first && found->rr_last);
 
 	/* remove from packet */
 	if(found_prev)	found_prev->rrset_all_next = found->rrset_all_next;
