@@ -199,7 +199,15 @@ comm_base_create(int sigs)
 	comm_base_now(b);
 	/* avoid event_get_method call which causes crashes even when
 	 * not printing, because its result is passed */
-	verbose(VERB_ALGO, "libevent %s uses %s method.", 
+	verbose(VERB_ALGO, 
+#ifdef HAVE_EV_LOOP
+		"libev"
+#elif defined(USE_MINI_EVENT)
+		"event "
+#else
+		"libevent "
+#endif
+		"%s uses %s method.", 
 		event_get_version(), 
 #ifdef HAVE_EVENT_BASE_GET_METHOD
 		event_base_get_method(b->eb->base)
@@ -325,7 +333,7 @@ comm_point_send_udp_msg(struct comm_point *c, ldns_buffer* packet,
 }
 
 /** print debug ancillary info */
-void p_ancil(const char* str, struct comm_reply* r)
+static void p_ancil(const char* str, struct comm_reply* r)
 {
 #if defined(AF_INET6) && defined(IPV6_PKTINFO) && (defined(HAVE_RECVMSG) || defined(HAVE_SENDMSG))
 	if(r->srctype != 4 && r->srctype != 6) {
@@ -372,7 +380,7 @@ void p_ancil(const char* str, struct comm_reply* r)
 }
 
 /** send a UDP reply over specified interface*/
-int
+static int
 comm_point_send_udp_msg_if(struct comm_point *c, ldns_buffer* packet,
 	struct sockaddr* addr, socklen_t addrlen, struct comm_reply* r) 
 {
@@ -474,7 +482,7 @@ comm_point_udp_ancil_callback(int fd, short event, void* arg)
 	struct comm_reply rep;
 	struct msghdr msg;
 	struct iovec iov[1];
-	ssize_t recv;
+	ssize_t rcv;
 	char ancil[256];
 	int i;
 #ifndef S_SPLINT_S
@@ -504,15 +512,15 @@ comm_point_udp_ancil_callback(int fd, short event, void* arg)
 		msg.msg_controllen = sizeof(ancil);
 #endif /* S_SPLINT_S */
 		msg.msg_flags = 0;
-		recv = recvmsg(fd, &msg, 0);
-		if(recv == -1) {
+		rcv = recvmsg(fd, &msg, 0);
+		if(rcv == -1) {
 			if(errno != EAGAIN && errno != EINTR) {
 				log_err("recvmsg failed: %s", strerror(errno));
 			}
 			return;
 		}
 		rep.addrlen = msg.msg_namelen;
-		ldns_buffer_skip(rep.c->buffer, recv);
+		ldns_buffer_skip(rep.c->buffer, rcv);
 		ldns_buffer_flip(rep.c->buffer);
 		rep.srctype = 0;
 #ifndef S_SPLINT_S
@@ -566,7 +574,7 @@ void
 comm_point_udp_callback(int fd, short event, void* arg)
 {
 	struct comm_reply rep;
-	ssize_t recv;
+	ssize_t rcv;
 	int i;
 
 	rep.c = (struct comm_point*)arg;
@@ -581,10 +589,10 @@ comm_point_udp_callback(int fd, short event, void* arg)
 		rep.addrlen = (socklen_t)sizeof(rep.addr);
 		log_assert(fd != -1);
 		log_assert(ldns_buffer_remaining(rep.c->buffer) > 0);
-		recv = recvfrom(fd, (void*)ldns_buffer_begin(rep.c->buffer), 
+		rcv = recvfrom(fd, (void*)ldns_buffer_begin(rep.c->buffer), 
 			ldns_buffer_remaining(rep.c->buffer), 0, 
 			(struct sockaddr*)&rep.addr, &rep.addrlen);
-		if(recv == -1) {
+		if(rcv == -1) {
 #ifndef USE_WINSOCK
 			if(errno != EAGAIN && errno != EINTR)
 				log_err("recvfrom %d failed: %s", 
@@ -598,7 +606,7 @@ comm_point_udp_callback(int fd, short event, void* arg)
 #endif
 			return;
 		}
-		ldns_buffer_skip(rep.c->buffer, recv);
+		ldns_buffer_skip(rep.c->buffer, rcv);
 		ldns_buffer_flip(rep.c->buffer);
 		rep.srctype = 0;
 		fptr_ok(fptr_whitelist_comm_point(rep.c->callback));
@@ -923,7 +931,8 @@ comm_point_tcp_handle_write(int fd, struct comm_point* c)
 		log_assert(iov[1].iov_len > 0);
 		r = writev(fd, iov, 2);
 #else /* HAVE_WRITEV */
-		r = send(fd, (void*)&len, sizeof(uint16_t), 0);
+		r = send(fd, (void*)(((uint8_t*)&len)+c->tcp_byte_count),
+			sizeof(uint16_t)-c->tcp_byte_count, 0);
 #endif /* HAVE_WRITEV */
 		if(r == -1) {
 #ifndef USE_WINSOCK
