@@ -135,7 +135,7 @@ void slabhash_status(struct slabhash* sl, const char* id, int extended)
 	size_t i;
 	char num[17];
 	log_info("Slabhash %s: %u tables mask=%x shift=%d", 
-		id, (unsigned)sl->size, sl->mask, sl->shift);
+		id, (unsigned)sl->size, (unsigned)sl->mask, sl->shift);
 	for(i=0; i<sl->size; i++) {
 		snprintf(num, sizeof(num), "table %u", (unsigned)i);
 		lruhash_status(sl->array[i], num, extended);
@@ -209,3 +209,11 @@ void slabhash_setmarkdel(struct slabhash* sl, lruhash_markdelfunc_t md)
 		lruhash_setmarkdel(sl->array[i], md);
 	}
 }
+
+void slabhash_traverse(struct slabhash* sh, int wr,
+        void (*func)(struct lruhash_entry*, void*), void* arg)
+{
+	size_t i;
+	for(i=0; i<sh->size; i++)
+		lruhash_traverse(sh->array[i], wr, func, arg);
+}
