@@ -40,6 +40,7 @@
  * According to RFC 4034.
  */
 #include "config.h"
+#include "ldns/ldns.h"
 #include "validator/validator.h"
 #include "validator/val_anchor.h"
 #include "validator/val_kcache.h"
@@ -1272,6 +1273,12 @@ processInit(struct module_qstate* qstate, struct val_qstate* vq,
 			return 1;
 		}
 		verbose(VERB_ALGO, "trust anchor NXDOMAIN by signed parent");
+	} else if(subtype == VAL_CLASS_POSITIVE &&
+		qstate->qinfo.qtype == LDNS_RR_TYPE_DNSKEY &&
+		query_dname_compare(lookup_name, qstate->qinfo.qname) == 0) {
+		/* is a DNSKEY so lookup a bit higher since we want to
+		 * get it from a parent or from trustanchor */
+		dname_remove_label(&lookup_name, &lookup_len);
 	}
 
 	if(vq->rrset_skip > 0 || subtype == VAL_CLASS_CNAME ||
@@ -1375,7 +1382,7 @@ static int
 processFindKey(struct module_qstate* qstate, struct val_qstate* vq, int id)
 {
 	uint8_t* target_key_name, *current_key_name;
-	size_t target_key_len, current_key_len;
+	size_t target_key_len;
 	int strip_lab;
 
 	log_query_info(VERB_ALGO, "validator: FindKey", &vq->qchase);
@@ -1404,7 +1411,6 @@ processFindKey(struct module_qstate* qstate, struct val_qstate* vq, int id)
 	}
 
 	current_key_name = vq->key_entry->name;
-	current_key_len = vq->key_entry->namelen;
 
 	/* If our current key entry matches our target, then we are done. */
 	if(query_dname_compare(target_key_name, current_key_name) == 0) {
@@ -1420,12 +1426,13 @@ processFindKey(struct module_qstate* qstate, struct val_qstate* vq, int id)
 			vq->empty_DS_name) == 0) {
 			/* do not query for empty_DS_name again */
 			verbose(VERB_ALGO, "Cannot retrieve DS for signature");
+			errinf(qstate, "no signatures");
+			errinf_origin(qstate, qstate->reply_origin);
 			vq->chase_reply->security = sec_status_bogus;
 			vq->state = VAL_FINISHED_STATE;
 			return 1;
 		}
 		current_key_name = vq->empty_DS_name;
-		current_key_len = vq->empty_DS_len;
 	}
 
 	log_nametypeclass(VERB_ALGO, "current keyname", current_key_name,
@@ -1732,7 +1739,7 @@ val_dlv_init(struct module_qstate* qstate, struct val_qstate* vq,
 	}
 	log_nametypeclass(VERB_ALGO, "DLV init look", nm, LDNS_RR_TYPE_DS,
 		vq->qchase.qclass);
-	log_assert(nm && nm_len)
+	log_assert(nm && nm_len);
 	/* sanity check: no DLV lookups below the DLV anchor itself.
 	 * Like, an securely insecure delegation there makes no sense. */
 	if(dname_subdomain_c(nm, qstate->env->anchors->dlv_anchor->name)) {
@@ -2452,10 +2459,46 @@ ds_response_to_ke(struct module_qstate* qstate, struct val_qstate* vq,
 			"bogus", val_classification_to_string(subtype));
 		errinf(qstate, "no DS but also no proof of that");
 		goto return_bogus;
+	} else if(subtype == VAL_CLASS_CNAME || 
+		subtype == VAL_CLASS_CNAMENOANSWER) {
+		/* if the CNAME matches the exact name we want and is signed
+		 * properly, then also, we are sure that no DS exists there,
+		 * much like a NODATA proof */
+		enum sec_status sec;
+		struct ub_packed_rrset_key* cname;
+		cname = reply_find_rrset_section_an(msg->rep, qinfo->qname,
+			qinfo->qname_len, LDNS_RR_TYPE_CNAME, qinfo->qclass);
+		if(!cname) {
+			errinf(qstate, "validator classified CNAME but no "
+				"CNAME of the queried name for DS");
+			goto return_bogus;
+		}
+		if(((struct packed_rrset_data*)cname->entry.data)->rrsig_count
+			== 0) {
+		        if(msg->rep->an_numrrsets != 0 && ntohs(msg->rep->
+				rrsets[0]->rk.type)==LDNS_RR_TYPE_DNAME) {
+				errinf(qstate, "DS got DNAME answer");
+			} else {
+				errinf(qstate, "DS got unsigned CNAME answer");
+			}
+			goto return_bogus;
+		}
+		sec = val_verify_rrset_entry(qstate->env, ve, cname, 
+			vq->key_entry, &reason);
+		if(sec == sec_status_secure) {
+			verbose(VERB_ALGO, "CNAME validated, "
+				"proof that DS does not exist");
+			/* and that it is not a referral point */
+			*ke = NULL;
+			return 1;
+		}
+		errinf(qstate, "CNAME in DS response was not secure.");
+		errinf(qstate, reason);
+		goto return_bogus;
 	} else {
 		verbose(VERB_QUERY, "Encountered an unhandled type of "
 			"DS response, thus bogus.");
-		errinf(qstate, "no DS and ");
+		errinf(qstate, "no DS and");
 		if(FLAGS_GET_RCODE(msg->rep->flags) != LDNS_RCODE_NOERROR) {
 			char* rc = ldns_pkt_rcode2str(
 				FLAGS_GET_RCODE(msg->rep->flags));
@@ -2881,7 +2924,7 @@ static struct module_func_block val_block = {
 };
 
 struct module_func_block* 
-val_get_funcblock()
+val_get_funcblock(void)
 {
 	return &val_block;
 }
